using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace VerticalParallax
{
    /// <summary>
    /// Vertical parallax example
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;                     // Game graphics device
        SpriteBatch spriteBatch;                            // spritebatch to writte 2D objects
        private Texture2D planet;                           // planet texture
        private Texture2D space;                            // background texture
        private Texture2D rocks;                            // Meteors texture
        private List<ParallaxLayer> listParallaxLayer;      // List of parallayx layers        
        bool manual;                                        // For this example, moving manually or auto scrolling?
        bool help;                                          // Screen help?
        SpriteFont font;                                    // Font used in screen help

        /// <summary>
        /// Game constructor
        /// </summary>
        public Game1()
        {
            // get graphics and content directory
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // auto scrolling and help enabled by default
            manual = false;
            help = true;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {       
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            // Load font
            font = Content.Load<SpriteFont>("gameFont");

            // Loading the parallax scrolling textures
            planet = this.Content.Load<Texture2D>("planet");
            rocks = this.Content.Load<Texture2D>("rocks");
            space = this.Content.Load<Texture2D>("space"); 

               // The order inserting the layers is important !!!!!!!!
            // first the closest
            listParallaxLayer = new List<ParallaxLayer>(); 
            listParallaxLayer.Add(new ParallaxLayer(graphics, space, 0.0f)); 
            listParallaxLayer.Add(new ParallaxLayer(graphics, planet, 10.0f));             
            listParallaxLayer.Add(new ParallaxLayer(graphics, rocks, 50.0f)); 
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();


            if (Keyboard.GetState().IsKeyDown(Keys.F2))            
                manual = !manual;

            if (Keyboard.GetState().IsKeyDown(Keys.F1))
                help = !help;
            
            
            //Mode 1: Auto scrolling
            if (!manual)
            {
                foreach (ParallaxLayer capa in listParallaxLayer)
                {
                    capa.MoveDown(gameTime);                    
                }
            }

            // Mode 2: manual scrolling, keyboard input
            else
            {
                if (Keyboard.GetState().IsKeyDown(Keys.Up))
                {
                    foreach (ParallaxLayer capa in listParallaxLayer)
                    {
                        capa.MoveDown(gameTime);
                    }
                }
                else
                {
                    if (Keyboard.GetState().IsKeyDown(Keys.Down))
                    {
                        foreach (ParallaxLayer capa in listParallaxLayer)
                        {
                            capa.MoveUp(gameTime);
                        }
                    }
                }
            } 

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            //Pintamos las capas
            foreach (ParallaxLayer capa in listParallaxLayer)
            {
                capa.Draw(spriteBatch);
            }

            if(help){
                spriteBatch.DrawString(font, "Manual / Scrolling mode F2", new Vector2(400,0), Color.White);
                spriteBatch.DrawString(font, "Screen help mode F1", new Vector2(600,50), Color.White);

                spriteBatch.DrawString(font, "Layer positions:" , new Vector2(500, 160), Color.White);
                spriteBatch.DrawString(font, "Space layer1 - " + listParallaxLayer[0].position1, new Vector2(500, 200), Color.White);
                spriteBatch.DrawString(font, "Space layer2 - " + listParallaxLayer[0].position2, new Vector2(500, 230), Color.White);
                spriteBatch.DrawString(font, "Rocks layer1 - "+ listParallaxLayer[1].position1, new Vector2(500,280), Color.White);
                spriteBatch.DrawString(font, "Rocks layer2 - " + listParallaxLayer[1].position2, new Vector2(500, 310), Color.White);
                spriteBatch.DrawString(font, "Meteors layer1 - " + listParallaxLayer[2].position1, new Vector2(500, 360), Color.White);
                spriteBatch.DrawString(font, "Meteors layer1 - " + listParallaxLayer[2].position2, new Vector2(500, 390), Color.White);

                if (manual)
                    spriteBatch.DrawString(font, "(Scrolling mode)", new Vector2(650, 0), Color.White);
                else
                    spriteBatch.DrawString(font, "(Manual mode)", new Vector2(650, 0), Color.White);

            }

            spriteBatch.End();
 
            base.Draw(gameTime);
        }
    }
}